@@ -8,7 +8,7 @@ from django.db import Error, models
 from django.utils.translation import gettext as _
 from esi.errors import TokenExpiredError, TokenInvalidError
 from esi.models import Token
-from eveuniverse.models import EveSolarSystem, EveType
+from eveuniverse.models import EveEntity, EveSolarSystem, EveType
 
 from allianceauth.authentication.models import CharacterOwnership, State
 
@@ -20,7 +20,6 @@ from buybackprogram.notification import (
     send_message_to_discord_channel,
     send_user_notification,
 )
-from buybackprogram.utils import get_site_url
 
 from .decorators import fetch_token_for_owner
 from .providers import esi
@@ -98,13 +97,15 @@ class Owner(models.Model):
     )
     def update_contracts_esi(self, token):
 
-        logger.debug("Fetching contracts for %s" % self.user)
+        logger.debug("Fetching contracts for %s" % self.character)
 
         # Get all contracts for owner
         contracts = self._fetch_contracts()
 
         logger.debug("Got %s character contracts" % len(contracts))
 
+        logger.debug("Fetching corporation contracts for %s" % self.corporation)
+
         # Get all contracts for owner corporation
         corporation_contracts = self._fetch_corporation_contracts()
 
@@ -118,193 +119,345 @@ class Owner(models.Model):
         # Get all tracking objects from the database
         tracking_numbers = Tracking.objects.all()
 
+        logger.debug("Got %s tracking numbers from database" % len(tracking_numbers))
+
         # Start looping for all stored tracking objects
         for tracking in tracking_numbers:
-            # Start checking if we find any matches from our ESI contracts
-            for contract in all_contracts:
+            logger.debug("Processing tracking number %s" % tracking.tracking_number)
 
-                # Only get contracts with the correct prefill ticker
-                if tracking.tracking_number in contract["title"]:
+            # If the tracking has an active program (not deleted)
+            if tracking.program:
 
-                    logger.debug(
-                        "Found a matching contract from tracking for %s "
-                        % contract["contract_id"]
-                    )
+                # Start checking if we find any matches from our ESI contracts
+                for contract in all_contracts:
 
-                    # Check if we already have the contract stored
-                    try:
-                        old_contract = Contract.objects.get(
-                            contract_id=contract["contract_id"]
+                    # Only get contracts with the correct prefill ticker
+                    if tracking.tracking_number in contract["title"]:
+
+                        logger.debug(
+                            "Found a matching tracking for %s "
+                            % contract["contract_id"]
                         )
 
-                    except Contract.DoesNotExist:
-                        logger.debug("No matching contracts found")
-                        old_contract = Contract.objects.none()
-                        old_contract.status = False
+                        # Check if we already have the contract stored
+                        try:
+                            old_contract = Contract.objects.get(
+                                contract_id=contract["contract_id"]
+                            )
 
-                    # If we have found a contract from database that is not yet finished
-                    if old_contract.status not in ["finished", "rejected"]:
-
-                        # Create or update the found contract
-                        obj, created = Contract.objects.update_or_create(
-                            contract_id=contract["contract_id"],
-                            defaults={
-                                "contract_id": contract["contract_id"],
-                                "assignee_id": contract["assignee_id"],
-                                "availability": contract["availability"],
-                                "date_completed": contract["date_completed"],
-                                "date_expired": contract["date_expired"],
-                                "date_issued": contract["date_issued"],
-                                "for_corporation": contract["for_corporation"],
-                                "issuer_corporation_id": contract[
-                                    "issuer_corporation_id"
-                                ],
-                                "issuer_id": contract["issuer_id"],
-                                "start_location_id": contract["start_location_id"],
-                                "price": contract["price"],
-                                "status": contract["status"],
-                                "title": contract["title"],
-                                "volume": contract["volume"],
-                            },
-                        )
-
-                        # If we have created a new contract
-                        if created:
                             logger.debug(
-                                "New contract %s created. Starting item fetch"
+                                "Contract %s is already stored in database"
                                 % contract["contract_id"]
                             )
 
-                            character_id = self.character.character.character_id
+                        except Contract.DoesNotExist:
+                            logger.debug(
+                                "No matching contracts stored for %s in database, new contract."
+                                % contract["contract_id"]
+                            )
+                            old_contract = Contract.objects.none()
+                            old_contract.status = False
 
-                            # Get all items in the contract
-                            contract_items = esi.client.Contracts.get_characters_character_id_contracts_contract_id_items(
-                                character_id=character_id,
-                                contract_id=contract["contract_id"],
-                                token=token.valid_access_token(),
-                            ).results()
-
-                            objs = []
-
-                            # Prepare objects for bulk create
-                            for item in contract_items:
-
-                                cont = Contract.objects.get(
-                                    contract_id=contract["contract_id"]
-                                )
-                                itm = EveType.objects.get(pk=item["type_id"])
-
-                                contract_item = ContractItem(
-                                    contract=cont,
-                                    eve_type=itm,
-                                    quantity=item["quantity"],
-                                )
-
-                                objs.append(contract_item)
-
-                            try:
-                                ContractItem.objects.bulk_create(objs)
-                                logger.debug(
-                                    "Succesfully added items for contract %s into database"
-                                    % contract["contract_id"]
-                                )
-                            except Error as e:
-                                logger.error(
-                                    "Error adding items for contract %s: %s"
-                                    % (contract["contract_id"], e)
-                                )
-
-                            """
-                            Notifications for users who have the notifications enabled
-                            """
-
-                            disablation_hint = "*You can disable these notifications from your program settings*"
-
-                            user_message = {
-                                "title": "New contract assigned",
-                                "description": "A new buyback contract {0} with a value of {1} ISK has been assigned to you.\n\n{2}".format(
-                                    tracking.tracking_number,
-                                    intcomma(int(contract["price"])),
-                                    disablation_hint,
-                                ),
-                                "footer": "AA Buyback Program",
-                            }
-
-                            # If tracking is active and we should send a message for our users
-                            if tracking.program.discord_dm_notification:
-
-                                send_user_notification(
-                                    user=self.user,
-                                    level="success",
-                                    message=user_message,
-                                )
-
-                            # Notifications for the discord channel
-                            if tracking.program.discord_channel_notification:
-                                send_message_to_discord_channel(
-                                    channel_id=tracking.program.discord_channel_notification,
-                                    message=user_message,
-                                )
-
-                        # If contract was updated instead of created
-                        else:
-                            logger.debug("Contract %s updated." % obj.contract_id)
-
-                        # Check if the contract status has changed from ongoing to finished.
-                        if (
-                            old_contract.status == "outstanding"
-                            and obj.status == "finished"
-                        ):
-                            # Get notification settings for the contract issuer
-                            user_settings = UserSettings.objects.get(
-                                user=tracking.issuer_user
+                        # If we have found a contract from database that is not yet finished
+                        if old_contract.status not in ["finished", "rejected"]:
+                            logger.debug(
+                                "Contract %s status is still pending, starting updates"
+                                % contract["contract_id"]
                             )
 
-                            # If user has not disabled notifications
-                            if user_settings.disable_notifications is False:
+                            # Create or update the found contract
+                            obj, created = Contract.objects.update_or_create(
+                                contract_id=contract["contract_id"],
+                                defaults={
+                                    "contract_id": contract["contract_id"],
+                                    "assignee_id": contract["assignee_id"],
+                                    "availability": contract["availability"],
+                                    "date_completed": contract["date_completed"],
+                                    "date_expired": contract["date_expired"],
+                                    "date_issued": contract["date_issued"],
+                                    "for_corporation": contract["for_corporation"],
+                                    "issuer_corporation_id": contract[
+                                        "issuer_corporation_id"
+                                    ],
+                                    "issuer_id": contract["issuer_id"],
+                                    "start_location_id": contract["start_location_id"],
+                                    "price": contract["price"],
+                                    "status": contract["status"],
+                                    "title": contract["title"],
+                                    "volume": contract["volume"],
+                                },
+                            )
 
-                                disablation_hint = "*You can disable these notifications from [My Settings]({0}/buybackprogram/user_settings_edit)*".format(
-                                    get_site_url()
+                            # If we have created a new contract
+                            if created:
+                                logger.debug(
+                                    "New contract %s has been created. Starting item fetch"
+                                    % contract["contract_id"]
                                 )
 
-                                # Check if the contract was accepted or rejected
-                                if contract["status"] == "finished":
-                                    status = "accepted"
-                                elif contract["status"] == "rejected":
-                                    status = "rejected"
+                                character_id = self.character.character.character_id
+
+                                corporation_id = self.character.character.corporation_id
+
+                                logger.debug(
+                                    "Fetching items for %s with character %s. Corporation contract: %s"
+                                    % (
+                                        contract["contract_id"],
+                                        character_id,
+                                        contract["is_corporation"],
+                                    )
+                                )
+
+                                if not contract["is_corporation"]:
+                                    logger.debug(
+                                        "Looking up items for %s via character endpoint"
+                                        % contract["contract_id"]
+                                    )
+                                    # Get all items in the contract
+                                    contract_items = esi.client.Contracts.get_characters_character_id_contracts_contract_id_items(
+                                        character_id=character_id,
+                                        contract_id=contract["contract_id"],
+                                        token=token.valid_access_token(),
+                                    ).results()
+
                                 else:
-                                    status = contract["status"]
+                                    logger.debug(
+                                        "Looking up items for %s via corporation endpoint"
+                                        % contract["contract_id"]
+                                    )
+                                    contract_items = esi.client.Contracts.get_corporations_corporation_id_contracts_contract_id_items(
+                                        corporation_id=corporation_id,
+                                        contract_id=contract["contract_id"],
+                                        token=token.valid_access_token(),
+                                    ).results()
+
+                                logger.debug(
+                                    "%s items found in contract %s"
+                                    % (len(contract_items), contract["contract_id"])
+                                )
+
+                                corporations = CharacterOwnership.objects.filter(
+                                    user=tracking.program.owner.user
+                                ).values_list("character__corporation_id", flat=True)
+
+                                logger.debug(
+                                    "Got corporations for contract owner: %s"
+                                    % corporations
+                                )
+
+                                objs = []
+
+                                # Prepare objects for bulk create
+                                for item in contract_items:
+
+                                    cont = Contract.objects.get(
+                                        contract_id=contract["contract_id"]
+                                    )
+                                    itm = EveType.objects.get(pk=item["type_id"])
+
+                                    contract_item = ContractItem(
+                                        contract=cont,
+                                        eve_type=itm,
+                                        quantity=item["quantity"],
+                                    )
+
+                                    objs.append(contract_item)
+
+                                try:
+                                    ContractItem.objects.bulk_create(objs)
+                                    logger.debug(
+                                        "Succesfully added %s items for contract %s into database"
+                                        % (len(objs), contract["contract_id"])
+                                    )
+                                except Error as e:
+                                    logger.error(
+                                        "Error adding items for contract %s: %s"
+                                        % (contract["contract_id"], e)
+                                    )
+
+                                # Check and see if any notifications/warnings should be set on the contract
+                                self._set_contract_notifications(
+                                    tracking, obj, corporations
+                                )
+
+                                # Notifications for users who have the notifications enabled
+
+                                if not contract["is_corporation"]:
+
+                                    assigned_to = (
+                                        self.character.character.character_name
+                                    )
+                                else:
+                                    assigned_to = (
+                                        self.character.character.corporation_name
+                                    )
+
+                                notifications = ContractNotification.objects.filter(
+                                    contract__contract_id=contract["contract_id"]
+                                )
+
+                                notes = str()
+
+                                if notifications:
+                                    notes += "\n\n**Notes**:\n\n"
+                                    for note in notifications:
+
+                                        notes += str(note.message)
+                                        notes += "\n\n"
 
                                 user_message = {
-                                    "title": "Buyback contract {0}".format(status),
-                                    "description": "Your outstanding buyback contract {0} with a value of {1} ISK has been {2} by {3}.\n\n{4}".format(
+                                    "title": "New buyback contract assigned",
+                                    "description": "A new buyback contract with tracking number {0} has been assigned to you.{1}".format(
                                         tracking.tracking_number,
-                                        intcomma(int(contract["price"])),
-                                        status,
-                                        self.character.character,
-                                        disablation_hint,
+                                        notes,
                                     ),
-                                    "footer": "AA Buyback Program",
+                                    "color": 0x5BC0DE,
+                                    "value": intcomma(int(contract["price"])),
+                                    "assigned_to": assigned_to,
+                                    "assigned_from": EveEntity.objects.resolve_name(
+                                        contract["issuer_id"]
+                                    ),
+                                    "footer": "Hint: You can disable these notifications from your program settings",
                                 }
 
-                                send_user_notification(
-                                    user=tracking.issuer_user,
-                                    level="success",
-                                    message=user_message,
+                                # If tracking is active and we should send a message for our users
+                                if tracking.program.discord_dm_notification:
+
+                                    send_user_notification(
+                                        user=self.user,
+                                        level="success",
+                                        message=user_message,
+                                    )
+                                else:
+                                    logger.debug(
+                                        "Program owner does not want DM notifications, passing"
+                                    )
+
+                                # Notifications for the discord channel
+                                if tracking.program.discord_channel_notification:
+                                    send_message_to_discord_channel(
+                                        channel_id=tracking.program.discord_channel_notification,
+                                        message=user_message,
+                                    )
+                                else:
+                                    logger.debug(
+                                        "Program owner does not want channel notifications, passing"
+                                    )
+
+                            # If contract was updated instead of created
+                            else:
+                                logger.debug(
+                                    "Contract %s is not new, updated the old contract."
+                                    % obj.contract_id
                                 )
 
-                    break  # If we have found a match from our ESI contracts wi will stop looping on the contracts
+                            # Check if the contract status has changed from ongoing to finished.
+                            if old_contract.status == "outstanding" and obj.status in [
+                                "finished",
+                                "rejected",
+                            ]:
+                                logger.debug(
+                                    "Contract %s has been completed. Status has changed to %s"
+                                    % (obj.contract_id, obj.status)
+                                )
+
+                                # Get notification settings for the contract issuer
+                                user_settings = UserSettings.objects.get(
+                                    user=tracking.issuer_user
+                                )
+
+                                # If user has not disabled notifications
+                                if user_settings.disable_notifications is False:
+
+                                    if not contract["is_corporation"]:
+
+                                        assigned_to = (
+                                            self.character.character.character_name
+                                        )
+                                    else:
+                                        assigned_to = (
+                                            self.character.character.corporation_name
+                                        )
+
+                                    # Check if the contract was accepted or rejected
+                                    if contract["status"] == "finished":
+                                        status = "accepted"
+                                        color = 0x5CB85C
+                                        level = "success"
+                                    elif contract["status"] == "rejected":
+                                        status = "rejected"
+                                        color = 0xD9534F
+                                        level = "danger"
+                                    else:
+                                        status = contract["status"]
+
+                                    notifications = ContractNotification.objects.filter(
+                                        contract__contract_id=contract["contract_id"]
+                                    )
+
+                                    notes = str()
+
+                                    if notifications:
+                                        notes += "\n\n**Notes**:\n\n"
+                                        for note in notifications:
+
+                                            notes += str(note.message)
+                                            notes += "\n\n"
+
+                                    user_message = {
+                                        "title": "Buyback contract {0}".format(status),
+                                        "description": "Your outstanding buyback contract {0} has been {1} by {2}.{3}".format(
+                                            tracking.tracking_number,
+                                            status,
+                                            self.character.character,
+                                            notes,
+                                        ),
+                                        "color": color,
+                                        "value": intcomma(int(contract["price"])),
+                                        "assigned_to": assigned_to,
+                                        "assigned_from": EveEntity.objects.resolve_name(
+                                            contract["issuer_id"]
+                                        ),
+                                        "footer": "Hint: You can disable these notifications from your buybackprogram settings",
+                                    }
+
+                                    send_user_notification(
+                                        user=tracking.issuer_user,
+                                        level=level,
+                                        message=user_message,
+                                    )
+                                else:
+                                    logger.debug(
+                                        "Contract assigner has notifications set to %s, passing"
+                                        % user_settings.disable_notifications
+                                    )
+                            else:
+                                logger.debug(
+                                    "No changes to the status of contract %s, update completed"
+                                    % obj.contract_id
+                                )
+
+                        break  # If we have found a match from our ESI contracts wi will stop looping on the contract
 
     @fetch_token_for_owner(["esi-contracts.read_character_contracts.v1"])
     def _fetch_contracts(self, token) -> list:
 
         character_id = self.character.character.character_id
 
-        contracts = esi.client.Contracts.get_characters_character_id_contracts(
+        esi_contracts = esi.client.Contracts.get_characters_character_id_contracts(
             character_id=character_id,
             token=token.valid_access_token(),
         ).results()
 
+        contracts = []
+
+        for esi_contract in esi_contracts:
+
+            contract = esi_contract
+            contract["is_corporation"] = False
+
+            contracts.append(contract)
+
         return contracts
 
     @fetch_token_for_owner(["esi-contracts.read_corporation_contracts.v1"])
@@ -312,13 +465,125 @@ class Owner(models.Model):
 
         corporation_id = self.character.character.corporation_id
 
-        contracts = esi.client.Contracts.get_corporations_corporation_id_contracts(
+        esi_contracts = esi.client.Contracts.get_corporations_corporation_id_contracts(
             corporation_id=corporation_id,
             token=token.valid_access_token(),
         ).results()
 
+        contracts = []
+
+        for esi_contract in esi_contracts:
+
+            contract = esi_contract
+            contract["is_corporation"] = True
+
+            contracts.append(contract)
+
         return contracts
 
+    def _set_contract_notifications(self, tracking, contract, corporations):
+
+        # List for all notes
+        notes = []
+
+        # Get structure id for tracked contract
+        structure_id = tracking.program.location.structure_id
+
+        # If our tracked price is different than the actual contract price
+        if tracking.net_price != contract.price:
+
+            # If contract price is bellow tracked price
+            if contract.price > tracking.net_price:
+
+                note = ContractNotification(
+                    contract=contract,
+                    icon="fa-dollar-sign",
+                    color="red",
+                    message="Ask price is above the calculated price for this contract",
+                )
+
+                notes.append(note)
+
+            else:
+
+                note = ContractNotification(
+                    contract=contract,
+                    icon="fa-dollar-sign",
+                    color="orange",
+                    message="Ask price is bellow the calculated price for this contract",
+                )
+
+                notes.append(note)
+
+        if structure_id and structure_id != contract.start_location_id:
+
+            note = ContractNotification(
+                contract=contract,
+                icon="fa-compass",
+                color="red",
+                message="Contract location does not match program location",
+            )
+
+            notes.append(note)
+
+        if contract.assignee_id in corporations and not tracking.program.is_corporation:
+
+            note = ContractNotification(
+                contract=contract,
+                icon="fa-home",
+                color="orange",
+                message="Contract is made for corporation while it should be made directly to the program managers character",
+            )
+
+            notes.append(note)
+
+        if contract.assignee_id not in corporations and tracking.program.is_corporation:
+
+            note = ContractNotification(
+                contract=contract,
+                icon="fa-user",
+                color="orange",
+                message="Contract is made for the program managers character while it should be made to the managers corporation",
+            )
+
+            notes.append(note)
+
+        if not tracking.tracking_number == contract.title:
+
+            note = ContractNotification(
+                contract=contract,
+                icon="fa-exclamation",
+                color="orange",
+                message="Contract description contains extra characterse besides the tracking number. The description should be: '%s', instead it is: '%s'"
+                % (tracking.tracking_number, contract.title),
+            )
+
+            notes.append(note)
+
+        if tracking.donation:
+
+            note = ContractNotification(
+                contract=contract,
+                icon="fa-hand-holding-usd",
+                color="green",
+                message="Contact contains a donation",
+            )
+
+            notes.append(note)
+
+        try:
+            ContractNotification.objects.bulk_create(notes)
+
+            logger.debug(
+                "Succesfully added items for contract %s into database"
+                % contract.contract_id
+            )
+
+        except Error as e:
+            logger.error(
+                "Error adding items for contract %s: %s" % (contract.contract_id, e)
+            )
+
     def token(self, scopes=None) -> Tuple[Token, int]:
         """returns a valid Token for the owner"""
         token = None
@@ -711,6 +976,26 @@ class ContractItem(models.Model):
     quantity = models.IntegerField()
 
 
+class ContractNotification(models.Model):
+
+    contract = models.ForeignKey(
+        Contract,
+        on_delete=models.deletion.CASCADE,
+    )
+
+    icon = models.CharField(
+        max_length=64,
+    )
+
+    color = models.CharField(
+        max_length=32,
+    )
+
+    message = models.CharField(
+        max_length=1024,
+    )
+
+
 class UserSettings(models.Model):
     """
     User settings
