@@ -11,8 +11,8 @@ from allianceauth.authentication.models import CharacterOwnership
 from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo
 from allianceauth.services.hooks import get_extension_logger
 
-from buybackprogram.forms import LocationForm, ProgramForm, ProgramItemForm
-from buybackprogram.models import Location, Owner, Program, ProgramItem
+from buybackprogram.forms import LocationForm, ProgramForm
+from buybackprogram.models import Location, Owner, Program
 from buybackprogram.utils import messages_plus
 
 logger = get_extension_logger(__name__)
@@ -167,23 +167,29 @@ def location_add(request):
     locations = Location.objects.all()
 
     if request.method != "POST":
-        form = LocationForm()
+        form = LocationForm(user=request.user)
     else:
         form = LocationForm(
-            request.POST,
-            value=int(request.POST["eve_solar_system"]),
+            request.POST, value=int(request.POST["eve_solar_system"]), user=request.user
         )
 
         if form.is_valid():
             eve_solar_system = form.cleaned_data["eve_solar_system"]
             name = form.cleaned_data["name"]
+            structure_id = form.cleaned_data["structure_id"] or None
+
+            owner = Owner.objects.filter(user=request.user).first()
 
             created = Location.objects.update_or_create(
                 eve_solar_system=eve_solar_system,
                 name=name,
+                structure_id=structure_id,
+                owner=owner,
                 defaults={
                     "eve_solar_system": eve_solar_system,
                     "name": name,
+                    "structure_id": structure_id,
+                    "owner": owner,
                 },
             )
 
@@ -208,53 +214,39 @@ def location_add(request):
 
 
 @login_required
-@permission_required("buybackprogram.basic_access")
-def program_edit_item(request, program_pk):
-    program = Program.objects.get(pk=program_pk)
+@permission_required("buybackprogram.manage_programs")
+def location_remove(request, location_pk):
 
-    program_items = ProgramItem.objects.filter(program=program)
+    location = Location.objects.get(pk=location_pk)
 
-    if request.method != "POST":
-        form = ProgramItemForm()
-    else:
-        form = ProgramItemForm(
-            request.POST,
-            value=int(request.POST["item_type"]),
+    if location.owner.user == request.user:
+        location.delete()
+
+        messages_plus.warning(
+            request,
+            format_html(
+                gettext_lazy(
+                    "Deleted location %(location)s. Any programs affiliated with this location has been removed as well."
+                )
+                % {
+                    "location": format_html(
+                        "<strong>{}</strong>", location.eve_solar_system
+                    ),
+                }
+            ),
         )
 
-        if form.is_valid():
-            item_type = form.cleaned_data["item_type"]
-            item_tax = form.cleaned_data["item_tax"]
-            disallow_item = form.cleaned_data["disallow_item"]
-
-            created = ProgramItem.objects.update_or_create(
-                item_type=item_type,
-                program=program,
-                defaults={
-                    "item_tax": item_tax,
-                    "disallow_item": disallow_item,
-                },
-            )
-
-            if created:
-                messages_plus.success(
-                    request,
-                    format_html(
-                        "Added <strong>{}</strong> to program with <strong>{}</strong> % tax",
-                        item_type,
-                        item_tax,
-                    ),
+    else:
+        messages_plus.error(
+            request,
+            format_html(
+                gettext_lazy(
+                    "You did not create this location and thus you can't delete it."
                 )
+            ),
+        )
 
-            return HttpResponseRedirect(request.path_info)
-
-    context = {
-        "program": program,
-        "program_items": program_items,
-        "form": form,
-    }
-
-    return render(request, "buybackprogram/program_edit_item.html", context)
+    return redirect("/buybackprogram/location_add")
 
 
 @login_required
@@ -287,24 +279,3 @@ def program_remove(request, program_pk):
         )
 
     return redirect("buybackprogram:index")
-
-
-@login_required
-@permission_required("buybackprogram.manage_programs")
-def program_item_remove(request, item_pk, program_pk):
-
-    program_item = ProgramItem.objects.get(item_type=item_pk)
-
-    name = program_item.item_type
-
-    program_item.delete()
-
-    messages_plus.warning(
-        request,
-        format_html(
-            "Deleted <strong>{}</strong> from program",
-            name,
-        ),
-    )
-
-    return redirect("buybackprogram:program_edit_item", program_pk)
